/* 
 
 MOTORS:
 Wiring setup for Mercury Motor 12V, 4 leads:
 
 Black - Terminal 1 - H Bridge 1Y
 Orange - Terminal 3 - H Bridge 2Y
 Brown - Terminal 2 - H Bridge 3Y
 Yellow - Terminal 4 - H Bridge 4Y
 
 COIN SENSOR:
 Normal value is around 40, but may vary.
 Presence of Coins over 80 or 90.
 K --> Ground
 A --> Anode (5V)
 C --> Vin (5V)
 E --> to Analog 1, branch to Ground through 1 MegaOhm resistor
 
 CROW SENSOR:
 normally above 1000, but may vary.
 presence of object reports number briefly below 100;
 
 
 MACHINE PHASES:
 1 - coins and peanuts fed onto tray when bird *leaves* the device
 2 - coins and peanuts fed onto tray when bird *arrives* at the device
 3 - coins fed onto tray when bird arrives, peanuts fed onto tray when coin falls into device
 4 - peanuts fed onto tray when coin falls into device
 
 */

#include <EEPROM.h>
#include <Metro.h>

///////////////////// PIN DECLARATIONS /////////////////////

//SENSORS
#define crow_sensor A0
#define coin_sensor A1


//MOTOR A - FOOD PEANUTS
int motorA_1 = 4;
int motorA_2 = 5;
int motorA_3 = 6;
int motorA_4 = 7;

//MOTOR B - COINS
int motorB_1 = 8;
int motorB_2 = 9;
int motorB_3 = 10;
int motorB_4 = 11;

//LEDs
int led_1 = 12;
int led_2 = 13;


///////////////////// VARIABLE DECLARATION /////////////////////

int motorA_index = 20; //number of steps to turn to initiate one action
int motorB_index = 17; //number of steps to turn to initiate one action

int the_delay = 8; //time delay between motor coil excitation
int the_delay2 = 8; //time delay between motor coil excitation

//EEPROM VARIABLES - Allow the machine to be unplugged and continue to operate where it left off once plugged in.
int the_phase = 1; //By default, set the phase to 1.
int phase_address = 1; //The EEPROM address containing the read/write location for the current phase of the machine.

int the_counter = 0; //By default, set the phase to 0.
int counter_address = 10; //The EEPROM address containing the read/write location for the counter of the machine.

int phase_2_threshold = 25; //the number of times the counter needs to tick in order to progress to the next phase.
int phase_3_threshold = 25; //the number of times the counter needs to tick in order to progress to the next phase.
int phase_4_threshold = 25; //the number of times the counter needs to tick in order to progress to the next phase.


///////////////////// SENSOR VALUES /////////////////////

// TUNE AS NECESSARY ::
int crow_threshold = 100; // MAY NEED TO TUNE THIS (from 0-1023) IF NOTICE THAT CROW SENSOR IS NOT ALWAYS RESPONDING
int coin_found = 3;      // time (in seconds) after coin sensed before another one may be sensed - MAY WANT TO INCREASE/DECREASE DEPENDING ON HOW MANY COINS, HOW FAST CROWS ARE ABLE TO BRING
int crow_gone = 5;      // time (in seconds) after crow sensed before dispensing in Phase 1 - MAY INCR/DECR FOR SLOWER/FASTER dispensing

////////////////////


// DO NOT EDIT ::

int crow_sensor_thresh = crow_threshold; //When it falls below this amount 

int found_interval = coin_found*1000; //allowed frequency of dispensing to happen, in milliseconds 
Metro found_timer = Metro(found_interval);

int crow_gone_interval = crow_gone*1000; //allotted time after the last crow is sensed before it dispenses in Phase 1.
Metro crow_gone_timer = Metro(crow_gone_interval);


int coin_sensor_val;
int crow_sensor_val;

int found_coins = 0;
int found_crow = 0;
int crow_here = 0;
int crow_left;

int input;

void setup() {

  Serial.begin(9600);

  pinMode(coin_sensor, INPUT);
  pinMode(crow_sensor, INPUT);

  pinMode(motorA_1, OUTPUT);
  pinMode(motorA_2, OUTPUT);
  pinMode(motorA_3, OUTPUT);
  pinMode(motorA_4, OUTPUT);

  pinMode(motorB_1, OUTPUT);
  pinMode(motorB_2, OUTPUT);
  pinMode(motorB_3, OUTPUT);
  pinMode(motorB_4, OUTPUT);
  
  pinMode(led_1, OUTPUT);
  pinMode(led_2, OUTPUT);
  
 
 //Simple startup sequence
   digitalWrite(led_1, HIGH); //blink lights to verify board is on.
   delay(500);
  digitalWrite(led_1, LOW);
   digitalWrite(led_2, HIGH); //blink lights to verify board is on.
   delay(500);
  digitalWrite(led_2, LOW);


  //Read the EEPROM for the current phase of the machine
  if (EEPROM.read(phase_address) == 2 || EEPROM.read(phase_address) == 3 || EEPROM.read(phase_address) == 4) {
    //IF the machine phase is set to 2, 3, or 4 before power-off, set it.
    int the_phase = EEPROM.read(phase_address);
  }

  if (EEPROM.read(counter_address) == 2 || EEPROM.read(phase_address) == 3 || EEPROM.read(phase_address) == 4) {
    //IF the machine phase is set to 2, 3, or 4 before power-off, set it.
    int the_phase = EEPROM.read(phase_address);
  }

} //END SETUP



void loop() {

  /////////////////////  SENSING AND TIMING ///////////////////////

  coin_sensor_val = digitalRead(coin_sensor);
  crow_sensor_val = analogRead(crow_sensor);



 if (coin_sensor_val == 1  && found_coins == 0) {  //if Coin IR sensor senses something and hasn't in the past 3 seconds
     digitalWrite(led_2, HIGH); //blink lights to verify board is on.
     found_timer.reset();
   
      found_coins = 1;
      Serial.println("COIN!"); 
   
      digitalWrite(led_2, LOW);
  
  }



  if (crow_sensor_val < crow_sensor_thresh && found_crow == 0 ) { //if Crow IR sensor senses something and hasn't for an interval of time
    
    digitalWrite(led_1, HIGH); //blink lights to verify board is on.
    Serial.println("CROW HERE!"); 
    delay(500);
    crow_gone_timer.reset();

    crow_here = 1;
    crow_left = 0;
    found_crow = 1;
    found_timer.reset();
    
     digitalWrite(led_1, LOW);
  }


  if (crow_gone_timer.check() == 1 && crow_here == 1 && crow_sensor_val > crow_sensor_thresh) { //the crow left
    Serial.println("CROW LEFT!"); 
    crow_left = 1;
    crow_here = 0;
    found_crow = 0;
  }


  if (found_timer.check() == 1) { //reset the found_coins trigger after the interval.
    found_coins = 0;
     
  }



  ///////////////////// MACHINE PHASE 1 - coins and peanuts fed onto tray when bird *leaves* the device ///////////////////////

  if (the_phase == 1) {
    if (crow_left == 1) { //if it has been long enough since the last crow is sensed
      Serial.println("PHASE 1 ACTION"); 
      dispense_food();
      dispense_coin();
      crow_left = 0;
      the_counter++; //increase the counter
      Serial.print("COUNT: ");
      Serial.println(the_counter);
    }
    if (the_counter == phase_2_threshold) { //Phase indexing
      the_phase = 2;
      EEPROM.write(phase_address,2); //update the phase in the EEPROM
      the_counter = 0;
    }   

  }



  ///////////////////// MACHINE PHASE 2 - coins and peanuts fed onto tray when bird *arrives* at the device ///////////////////////
  if (the_phase == 2) {
    if (found_crow == 1 && crow_left == 0) {
      Serial.println("PHASE 2 ACTION"); 
      dispense_food();
      dispense_coin();
      //found_crow = 0;
      crow_left = 1;    //using crow_left marker for 'crow-gone' time interval, to only dispense once per crow gathering
      the_counter++; //increase the counter
      Serial.print("COUNT: ");
      Serial.println(the_counter);
    }

    if (the_counter == phase_3_threshold) { //Phase indexing
      the_phase = 3;
      EEPROM.write(phase_address,3); //update the phase in the EEPROM
      the_counter = 0; 
    }
  }




  ///////////////// MACHINE PHASE 3 - coins fed onto tray when bird arrives, peanuts fed onto tray when coin falls into device //////
  if (the_phase == 3) {
    if (found_coins == 1) {
      Serial.println("PHASE 3 ACTION"); 
      dispense_food();
      found_coins = 0;  //to stop multiple dispensing during time interval, but each coin should get more peanuts
    }

    if (found_crow == 1 && crow_left == 0) {  //if crow arrives after a crow has left
      Serial.println("PHASE 3 ACTION"); 
      dispense_coin();
      //found_crow = 0;      //to stop multiple dispensing during time interval
      crow_left = 1;        //use crow_left marker for time interval, only dispenses once per crow gathering session
      the_counter++; //increase the counter
      Serial.print("COUNT: ");
      Serial.println(the_counter);
    }

    if (the_counter == phase_4_threshold) { //Phase indexing
      the_phase = 4;
      EEPROM.write(phase_address,4); //update the phase in the EEPROM
      the_counter = 0;
    }
  }




  ///////////////////// MACHINE PHASE 4 - peanuts fed onto tray when coin falls into device ///////////////////////
  if (the_phase == 4) {
    if (found_coins == 1) {
      Serial.println("PHASE 4 ACTION"); 
      dispense_food();
      found_coins = 0;
      the_counter++;
      Serial.print("COUNT: ");
      Serial.println(the_counter);
    }
  }



} //END LOOP!







int dispense_food() {
    digitalWrite(led_1, HIGH); 
  Serial.println("Dispensing FOOD"); 
  for (int i=1; i<motorA_index; i++){
    motorA_forward();
  }
  motorA_release();
   for (int i=1; i<motorA_index; i++){
    motorA_backward();
  }
  motorA_release();
    digitalWrite(led_1, LOW);
}

int dispense_coin() {
    digitalWrite(led_2, HIGH); 
  Serial.println("Dispensing COIN"); 
  for (int i=1; i<motorB_index; i++){
    motorB_backward();
  }
  motorB_release();
    digitalWrite(led_2, LOW);
}


int motorA_forward() {
  digitalWrite(motorA_1, HIGH);
  digitalWrite(motorA_2, LOW);
  digitalWrite(motorA_3, LOW);
  digitalWrite(motorA_4, HIGH);
  delay(the_delay2);
  digitalWrite(motorA_1, HIGH);
  digitalWrite(motorA_2, HIGH);
  digitalWrite(motorA_3, LOW);
  digitalWrite(motorA_4, LOW);
  delay(the_delay2);
  digitalWrite(motorA_1, LOW);
  digitalWrite(motorA_2, HIGH);
  digitalWrite(motorA_3, HIGH);
  digitalWrite(motorA_4, LOW);
  delay(the_delay2);
  digitalWrite(motorA_1, LOW);
  digitalWrite(motorA_2, LOW);
  digitalWrite(motorA_3, HIGH);
  digitalWrite(motorA_4, HIGH);
  delay(the_delay2);
}


int motorB_forward() {
  digitalWrite(motorB_1, HIGH);
  digitalWrite(motorB_2, LOW);
  digitalWrite(motorB_3, LOW);
  digitalWrite(motorB_4, HIGH);
  delay(the_delay);
  digitalWrite(motorB_1, HIGH);
  digitalWrite(motorB_2, HIGH);
  digitalWrite(motorB_3, LOW);
  digitalWrite(motorB_4, LOW);
  delay(the_delay);
  digitalWrite(motorB_1, LOW);
  digitalWrite(motorB_2, HIGH);
  digitalWrite(motorB_3, HIGH);
  digitalWrite(motorB_4, LOW);
  delay(the_delay);
  digitalWrite(motorB_1, LOW);
  digitalWrite(motorB_2, LOW);
  digitalWrite(motorB_3, HIGH);
  digitalWrite(motorB_4, HIGH);
  delay(the_delay);
}


int motorA_backward() { 
  digitalWrite(motorA_1, LOW);
  digitalWrite(motorA_2, HIGH);
  digitalWrite(motorA_3, HIGH);
  digitalWrite(motorA_4, LOW);
  delay(the_delay2);
  digitalWrite(motorA_1, HIGH);
  digitalWrite(motorA_2, HIGH);
  digitalWrite(motorA_3, LOW);
  digitalWrite(motorA_4, LOW);
  delay(the_delay2);
   digitalWrite(motorA_1, HIGH);
  digitalWrite(motorA_2, LOW);
  digitalWrite(motorA_3, LOW);
  digitalWrite(motorA_4, HIGH);
  delay(the_delay2);
    digitalWrite(motorA_1, LOW);
  digitalWrite(motorA_2, LOW);
  digitalWrite(motorA_3, HIGH);
  digitalWrite(motorA_4, HIGH);
  delay(the_delay2);
}


int motorB_backward() {
  digitalWrite(motorB_1, HIGH);
  digitalWrite(motorB_2, LOW);
  digitalWrite(motorB_3, LOW);
  digitalWrite(motorB_4, HIGH);
  delay(the_delay);
  digitalWrite(motorB_1, LOW);
  digitalWrite(motorB_2, LOW);
  digitalWrite(motorB_3, HIGH);
  digitalWrite(motorB_4, HIGH);
  delay(the_delay);
  digitalWrite(motorB_1, LOW);
  digitalWrite(motorB_2, HIGH);
  digitalWrite(motorB_3, HIGH);
  digitalWrite(motorB_4, LOW);
  delay(the_delay);
  digitalWrite(motorB_1, HIGH);
  digitalWrite(motorB_2, HIGH);
  digitalWrite(motorB_3, LOW);
  digitalWrite(motorB_4, LOW);
  delay(the_delay);
}


int motor_both_backward() {
  digitalWrite(motorB_1, HIGH);
  digitalWrite(motorB_2, LOW);
  digitalWrite(motorB_3, LOW);
  digitalWrite(motorB_4, HIGH);
  digitalWrite(motorA_1, HIGH);
  digitalWrite(motorA_2, LOW);
  digitalWrite(motorA_3, LOW);
  digitalWrite(motorA_4, HIGH);
  delay(the_delay);
  digitalWrite(motorB_1, LOW);
  digitalWrite(motorB_2, LOW);
  digitalWrite(motorB_3, HIGH);
  digitalWrite(motorB_4, HIGH);
  digitalWrite(motorA_1, LOW);
  digitalWrite(motorA_2, LOW);
  digitalWrite(motorA_3, HIGH);
  digitalWrite(motorA_4, HIGH);
  delay(the_delay);
  digitalWrite(motorB_1, LOW);
  digitalWrite(motorB_2, HIGH);
  digitalWrite(motorB_3, HIGH);
  digitalWrite(motorB_4, LOW);
  digitalWrite(motorA_1, LOW);
  digitalWrite(motorA_2, HIGH);
  digitalWrite(motorA_3, HIGH);
  digitalWrite(motorA_4, LOW);
  delay(the_delay);
  digitalWrite(motorB_1, HIGH);
  digitalWrite(motorB_2, HIGH);
  digitalWrite(motorB_3, LOW);
  digitalWrite(motorB_4, LOW);
  digitalWrite(motorA_1, HIGH);
  digitalWrite(motorA_2, HIGH);
  digitalWrite(motorA_3, LOW);
  digitalWrite(motorA_4, LOW);
  delay(the_delay);
}



int motorA_release() {
  digitalWrite(motorA_1, LOW);
  digitalWrite(motorA_2, LOW);
  digitalWrite(motorA_3, LOW);
  digitalWrite(motorA_4, LOW); 
}


int motorB_release() {
  digitalWrite(motorB_1, LOW);
  digitalWrite(motorB_2, LOW);
  digitalWrite(motorB_3, LOW);
  digitalWrite(motorB_4, LOW); 
}









